﻿#region Copyright (C) 2005-2012 Team MediaPortal

// Copyright (C) 2005-2011 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Windows.Forms;
using iSpyPlugin.Configuration;

namespace iSpyPlugin
{
    public partial class Configure : Form
    {
        public Configure()
        {
            InitializeComponent();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            var cs = new ConfigureServer();
            cs.ShowDialog(this);
            if (cs.DialogResult == DialogResult.OK)
            {
                if (ServerList.Servers.ispy.SingleOrDefault(p => p.ipaddress == cs.IP) == null)
                {
                    var l = ServerList.Servers.ispy.ToList();
                    IPAddress ip;
                    if (!IPAddress.TryParse(cs.IP, out ip))
                        MessageBox.Show("Not a valid IP address");
                    else
                    {
                        l.Add(new configIspy { ipaddress = cs.IP, password = "", port = cs.Port });
                        ServerList.Servers.ispy = l.ToArray();
                        LoadServers();
                        lbServers.SelectedIndex = lbServers.Items.Count - 1;
                    }
                    
                }
                else
                    MessageBox.Show("You have already added this IP address.");
            }
            cs.Dispose();
        }

        

        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (lbServers.SelectedIndex > -1)
            {
                var s = ServerList.Servers.ispy[lbServers.SelectedIndex];
                var cs = new ConfigureServer {IP = s.ipaddress, Port = s.port};

                cs.ShowDialog(this);
                if (cs.DialogResult == DialogResult.OK)
                {
                    ServerList.Servers.ispy[lbServers.SelectedIndex].ipaddress = cs.IP;
                    ServerList.Servers.ispy[lbServers.SelectedIndex].port = cs.Port;

                }
                cs.Dispose();              
                LoadServers();
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (lbServers.SelectedIndex > -1)
            {
                var l = ServerList.Servers.ispy.ToList();
                l.RemoveAt(lbServers.SelectedIndex);
                ServerList.Servers.ispy = l.ToArray();
                LoadServers();
            }
        }

        private void Configure_Load(object sender, EventArgs e)
        {
            LoadServers();
        }

        

        private void LoadServers()
        {
            lbServers.Items.Clear();

            foreach(var s in ServerList.Servers.ispy)
            {
                lbServers.Items.Add(s.ipaddress + ":" + s.port);
            }
            
            if (lbServers.Items.Count > 0)
                lbServers.SelectedIndex = 0;
            else
            {
                btnEdit.Enabled = btnRemove.Enabled = false;
            }
        }

        private void lbServers_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnEdit.Enabled = btnRemove.Enabled = lbServers.SelectedIndex > -1;
        }

        private void Configure_FormClosing(object sender, FormClosingEventArgs e)
        {
           ServerList.SaveConfig();
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.ispyconnect.com/download.aspx");            
        }
    }
}
