﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MediaPortal.GUI.Library;
using MediaPortal.Threading;
using MediaPortal.Common.Utils;

namespace iSpyPlugin
{
  public class Streamer : GUIWindow
  {
    public int cameraID = 0;

    [SkinControl(500)]
    private readonly GUIImage _videoPlayer0 = null;
    [SkinControl(501)]
    private readonly GUIImage _videoPlayer1 = null;
    [SkinControl(502)]
    private readonly GUIImage _videoPlayer2 = null;
    [SkinControl(503)]
    private readonly GUIImage _videoPlayer3 = null;

    private Int64 _framecount;
    private MJPEGStream _streamer = new MJPEGStream();

    public void start(string url)
    {
      _streamer = new MJPEGStream(url) { RequestTimeout = 4000 };
      _streamer.NewFrame += StreamerNewFrame;
      _streamer.VideoSourceError += StreamerVideoSourceError;
      _streamer.PlayingFinished += StreamerPlayingFinished;
      _streamer.Start();
      Log.Error("Start camera grid: " + cameraID);
    }

    void StreamerPlayingFinished(object sender, ReasonToFinishPlaying reason)
    {
      if (cameraID == 0)
      {
        _videoPlayer0.SetFileName("[ispy:offline]");
        _videoPlayer0.UpdateLayout();
      }
      if (cameraID == 1)
      {
        _videoPlayer1.SetFileName("[ispy:offline]");
        _videoPlayer1.UpdateLayout();
      }
      if (cameraID == 2)
      {
        _videoPlayer2.SetFileName("[ispy:offline]");
        _videoPlayer2.UpdateLayout();
      }
      if (cameraID == 3)
      {
        _videoPlayer3.SetFileName("[ispy:offline]");
        _videoPlayer3.UpdateLayout();
      }
    }

    void StreamerVideoSourceError(object sender, VideoSourceErrorEventArgs eventArgs)
    {
      lock (this)
      {
        try
        {

          if (cameraID == 0)
          {
            GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
            _videoPlayer0.SetFileName("[ispy:offline]");
            _videoPlayer0.UpdateLayout();
          }
          if (cameraID == 1)
          {
            GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
            _videoPlayer1.SetFileName("[ispy:offline]");
            _videoPlayer1.UpdateLayout();
          }
          if (cameraID == 2)
          {
            GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
            _videoPlayer2.SetFileName("[ispy:offline]");
            _videoPlayer2.UpdateLayout();
          }
          if (cameraID == 3)
          {
            GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
            _videoPlayer3.SetFileName("[ispy:offline]");
            _videoPlayer3.UpdateLayout();
          }

        }
        catch (Exception e)
        {
          Log.Error("Error occured in StreamerVideoSourceError");
          Log.Error(e.Message);
        }
      }
    }

    void StreamerNewFrame(object sender, NewFrameEventArgs eventArgs)
    {
      lock (this)
      {
        Log.Error("Setting filename: " + "[ispy:video" + _framecount + "]");

        if (cameraID == 0)
        {
          if (_framecount > 0)
          {
            _videoPlayer0.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
          }
          _framecount++;
          try
          {
            GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
              eventArgs.Frame.Height);
          }
          catch
          {

          }
          _videoPlayer0.SetFileName("[ispy:video" + _framecount + "]");
          _videoPlayer0.UpdateLayout();
          Log.Error("Setting filename: " + "[ispy:video" + _framecount + "]");
        }
        if (cameraID == 1)
        {
          if (_framecount > 0)
          {
            _videoPlayer1.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
          }
          _framecount++;
          try
          {
            GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
              eventArgs.Frame.Height);
          }
          catch
          {

          }
          _videoPlayer1.SetFileName("[ispy:video" + _framecount + "]");
          _videoPlayer1.UpdateLayout();
          Log.Error("Setting filename: "+ "[ispy:video" + _framecount + "]");
        }
        if (cameraID == 2)
        {
          if (_framecount > 0)
          {
            _videoPlayer2.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
          }
          _framecount++;
          try
          {
            GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
              eventArgs.Frame.Height);
          }
          catch
          {

          }
          _videoPlayer2.SetFileName("[ispy:video" + _framecount + "]");
          _videoPlayer2.UpdateLayout();
          Log.Error("Setting filename: " + "[ispy:video" + _framecount + "]");
        }
        if (cameraID == 3)
        {
          if (_framecount > 0)
          {
            _videoPlayer3.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
          }
          _framecount++;
          try
          {
            GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
              eventArgs.Frame.Height);
          }
          catch
          {

          }
          _videoPlayer3.SetFileName("[ispy:video" + _framecount + "]");
          _videoPlayer3.UpdateLayout();
          Log.Error("Setting filename: " + "[ispy:video             " + _framecount + "]");
        }
      }
    }
  }
}
