﻿using System;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace iSpyPlugin.Utility
{
    class Tools
    {
        private static readonly string NL = Environment.NewLine;
        private static IPAddress[] _ipv4Addresses;
        private static string _ipv4Address = "";


        public static string Shorten(string inStr, int max)
        {
            if (inStr.Length > max)
                inStr = inStr.Substring(0, max) + "...";
            return inStr;
        }

        public static string AddressIPv4
        {
            get
            {
                if (_ipv4Address != "")
                    return _ipv4Address;

                string detectip = "";
                foreach (IPAddress ip in AddressListIPv4)
                {
                    if (detectip == "")
                        detectip = ip.ToString();

                    if (ip.AddressFamily == AddressFamily.InterNetwork)
                    {

                        if (!IPAddress.IsLoopback(ip))
                        {
                            if (detectip == "")
                                detectip = ip.ToString();
                        }
                    }
                }
                if (_ipv4Address == "")
                    _ipv4Address = detectip;
                
                return _ipv4Address;
            }
            set { _ipv4Address = value; }
        }

        public static IPAddress[] AddressListIPv4
        {
            get
            {
                if (_ipv4Addresses != null)
                    return _ipv4Addresses;
                _ipv4Addresses = Dns.GetHostEntry(Dns.GetHostName()).AddressList.Where(p => p.AddressFamily == AddressFamily.InterNetwork).ToArray();
                return _ipv4Addresses;
            }
        }

        public static string GetFullUrlContent(string url, int timeoutSeconds)
        {
            Encoding enc = Encoding.GetEncoding("utf-8");
            string strResponse = null;
            HttpWebResponse response = null;
            try
            {
                var req = (HttpWebRequest)WebRequest.Create(url.Replace(" ", "-"));
                req.Method = "GET";
                req.Timeout = timeoutSeconds * 1000;
                response = (HttpWebResponse)req.GetResponse();
                Stream receiveStream = response.GetResponseStream();
                if (receiveStream != null)
                {
                    var readStream = new StreamReader(receiveStream, enc);
                    strResponse = readStream.ReadToEnd();
                    response.Close();
                    readStream.Close();
                    readStream.Dispose();
                }
                if (receiveStream != null)
                {
                    receiveStream.Close();
                    receiveStream.Dispose();
                }
            }
            catch (Exception ex)
            {
                strResponse = ex.Message;
                if (response != null)
                {
                    strResponse += NL + NL + "Status: " + response.StatusCode + ": " + response.StatusDescription;
                }
            }
            return strResponse;
        }

        public static byte[] GetBytesFromUrl(string url)
        {
            byte[] b = null;
            var myReq = (HttpWebRequest)WebRequest.Create(url);
            var myResp = myReq.GetResponse();

            var stream = myResp.GetResponseStream();

            if (stream != null)
                using (var br = new BinaryReader(stream))
                {
                    b = br.ReadBytes(1000000);
                    br.Close();
                }
            myResp.Close();
            return b;
        }

        private static Image ByteArrayToImage(byte[] byteArrayIn)
        {
            var ms = new MemoryStream(byteArrayIn);
            var returnImage = Image.FromStream(ms);
            return returnImage;
        }

        public static Image GetImageFromUrl(string url)
        {
            return ByteArrayToImage(GetBytesFromUrl(url));
        }


        /// <summary>
        /// Check if the array contains needle at specified position.
        /// </summary>
        /// 
        /// <param name="array">Source array to check for needle.</param>
        /// <param name="needle">Needle we are searching for.</param>
        /// <param name="startIndex">Start index in source array.</param>
        /// 
        /// <returns>Returns <b>true</b> if the source array contains the needle at
        /// the specified index. Otherwise it returns <b>false</b>.</returns>
        /// 
        public static bool Compare(byte[] array, byte[] needle, int startIndex)
        {
            int needleLen = needle.Length;
            // compare
            for (int i = 0, p = startIndex; i < needleLen; i++, p++)
            {
                if (array[p] != needle[i])
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Find subarray in the source array.
        /// </summary>
        /// 
        /// <param name="array">Source array to search for needle.</param>
        /// <param name="needle">Needle we are searching for.</param>
        /// <param name="startIndex">Start index in source array.</param>
        /// <param name="sourceLength">Number of bytes in source array, where the needle is searched for.</param>
        /// 
        /// <returns>Returns starting position of the needle if it was found or <b>-1</b> otherwise.</returns>
        /// 
        public static int Find(byte[] array, byte[] needle, int startIndex, int sourceLength)
        {
            int needleLen = needle.Length;
            int index;

            while (sourceLength >= needleLen)
            {
                // find needle's starting element
                index = Array.IndexOf(array, needle[0], startIndex, sourceLength - needleLen + 1);

                // if we did not find even the first element of the needls, then the search is failed
                if (index == -1)
                    return -1;

                int i, p;
                // check for needle
                for (i = 0, p = index; i < needleLen; i++, p++)
                {
                    if (array[p] != needle[i])
                    {
                        break;
                    }
                }

                if (i == needleLen)
                {
                    // needle was found
                    return index;
                }

                // continue to search for needle
                sourceLength -= (index - startIndex + 1);
                startIndex = index + 1;
            }
            return -1;
        }


        public static bool ValidateRemoteCertificate(
            object sender,
                X509Certificate certificate,
                X509Chain chain,
                SslPolicyErrors policyErrors
            )
        {
            //if (Convert.ToBoolean(ConfigurationManager.AppSettings["IgnoreSslErrors"]))
            //{
            // allow any old dodgy certificate...
            return true;
            //}
            //else
            //{
            //    return policyErrors == SslPolicyErrors.None;
            //}
        }
    }
}
