﻿#region Copyright (C) 2005-2012 Team MediaPortal

// Copyright (C) 2005-2011 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using iSpyPlugin.Configuration;
using MediaPortal.Dialogs;
using MediaPortal.ExtensionMethods;
using MediaPortal.GUI.Library;
using Action = MediaPortal.GUI.Library.Action;

namespace iSpyPlugin
{
  public class ispy_cameras : GUIWindow
  {
    private const int WindowId = 157801;
    private configIspy _server = null;

    public configIspy Server
    {
      get
      {
        return _server ?? (_server = ServerList.Servers.ispy.SingleOrDefault(
          p => p.ipaddress == GUIPropertyManager.GetProperty("#ispyserver")));
      }
      set { _server = value; }
    }

    private MJPEGStream _streamer;
    private Content.Camera _selectedCamera;
    private Int64 _framecount;
    const int MessageBoxId = (int) GUIWindow.Window.WINDOW_DIALOG_OK;

    /// <summary>
    /// The facade layout containing cameras.
    /// </summary>
    [SkinControl(50)] private readonly GUIFacadeControl _facadeLayout = null;

    [SkinControl(500)] private readonly GUIImage _videoPlayer = null;

    [SkinControl(2)] private readonly GUIButtonControl _govideo = null;

    [SkinControl(3)] private readonly GUIButtonControl _cmdrecord = null;

    [SkinControl(4)] private readonly GUIButtonControl _cmdstoprecord = null;

    [SkinControl(5)] private readonly GUIButtonControl _cmdenable = null;

    [SkinControl(6)] private readonly GUIButtonControl _cmddisable = null;

    public override bool Init()
    {

      return Load(GUIGraphicsContext.Skin + @"\ispy_cameras.xml");
    }

    protected override void OnPreviousWindow()
    {
      Cleanup();
      base.OnPreviousWindow();
    }

    private void Cleanup()
    {
      try
      {
        if (_streamer != null)
        {
          _streamer.SignalToStop();
          _streamer.WaitForStop();
          _streamer = null;
          _streamer.SafeDispose();
        }
      }
      catch (Exception e)
      {
        Log.Error("Error during cleanup");
        Log.Error(e.Message);
      }
    }

    protected override void OnPageLoad()
    {
      _server = null;
      if (ServerList.Servers.ispy.Length > 0)
      {
        _server = ServerList.Servers.ispy[0];
      }

      bool passed = false;
      var cams = new List<Content.Camera>();
      Cleanup();
      try
      {
        cams = Content.Cameras(Server);
        if (cams.Count == 0)
        {
          using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(MessageBoxId))
          {
            messageBox.SetHeading("No cameras found at " + Server.ipaddress);
            messageBox.SetLines("To add cameras, add them to iSpy and then try again.");
            messageBox.DoModal(GUIWindowManager.ActiveWindow);
          }
        }
        else
        {
          passed = true;
        }
      }
      catch (Exception ex)
      {
        using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(MessageBoxId))
        {
          messageBox.SetHeading("Error accessing iSpy at " + Server.ipaddress);
          messageBox.SetLines("Check your IP address and access permissions and try again.\n" +
                              ex.Message);
          messageBox.DoModal(GUIWindowManager.ActiveWindow);
        }
      }
      if (passed)
      {
        GUIWaitCursor.Init();
        GUIWaitCursor.Show();

        // Add servers
        foreach (var camera in cams)
        {
          _facadeLayout.Add(CreateListItemFrom(camera));
        }

        int selectedListItemIndex = _facadeLayout.SelectedListItemIndex;

        if (selectedListItemIndex != -1)
        {
          _selectedCamera = ((Content.Camera) _facadeLayout[selectedListItemIndex].TVTag);
          if (_selectedCamera.Online && !string.IsNullOrEmpty(_selectedCamera.IPAddress))
          {
            Cleanup();
            PlayLiveViewFromSelectedCamera();
          }
        }

        /*
        SetSelectedCamera();
        PlayLiveViewFromSelectedCamera();
        */
        GUIWaitCursor.Hide();
      }

      base.OnPageLoad();
    }

    protected override void OnPageDestroy(int newWindowId)
    {
      base.OnPageDestroy(newWindowId);
    }

    public override void Process()
    {
      lock (this)
      {
        base.Process();
      }
    }

    public override void DeInit()
    {
      Cleanup();
      base.DeInit();
    }

    /// <summary>
    /// Sets the selected server.
    /// </summary>
    private void SetSelectedCamera()
    {
      for (int index = 0; index < _facadeLayout.Count; index++)
      {
        var camera = (Content.Camera) _facadeLayout[index].TVTag;

        if (camera.Ident == _selectedCamera.Ident)
        {
          // Set the last selected camera
          GUIControl.SelectItemControl(GetID, _facadeLayout.GetID, index);
          break;
        }
      }

    }

    /// <summary>
    /// Creates a list item representing a camera.
    /// </summary>
    /// <param name="camera">The camera.</param>
    /// <returns>A list item representing a camera.</returns>
    private GUIListItem CreateListItemFrom(Content.Camera camera)
    {
      var listItem = new GUIListItem
      {
        Label = camera.Name,
        TVTag = camera
      };
      var img =
        Utility.Tools.GetImageFromUrl("http://" + Server.ipaddress + ":" + Server.port + "/livefeed?oid=" + camera.Id +
                                      "&size=32x32");
      if (img != null)
      {
        GUITextureManager.LoadFromMemory(img, "[" + camera.Ident + "]", 0x00000000, 32, 32);
        listItem.IconImage = "[" + camera.Ident + "]";
      }
      listItem.OnItemSelected += OnItemSelected;

      return listItem;
    }

    /// <summary>
    /// Called when list item is selected.
    /// </summary>
    /// <param name="item">list item.</param>
    /// <param name="parent">control parent.</param>
    private void OnItemSelected(GUIListItem item, GUIControl parent)
    {
      try
      {
      int selectedListItemIndex = _facadeLayout.SelectedListItemIndex;

      if (selectedListItemIndex != -1)
      {
        _selectedCamera = ((Content.Camera) _facadeLayout[selectedListItemIndex].TVTag);
        if (_selectedCamera.Online && !string.IsNullOrEmpty(_selectedCamera.IPAddress))
        {
            Cleanup();
            PlayLiveViewFromSelectedCamera();
        }
      }
      }
      catch (Exception e)
      {
        Log.Error("Error during OnItemSelected");
        Log.Error(e.Message);
      }
    }

    /// <summary>
    /// Called when control was clicked.
    /// </summary>
    /// <param name="controlId">The control id.</param>
    /// <param name="control">The control.</param>
    /// <param name="actionType">Type of the action.</param>
    protected override void OnClicked(int controlId, GUIControl control, Action.ActionType actionType)
    {
      base.OnClicked(controlId, control, actionType);

      if (actionType == Action.ActionType.ACTION_SELECT_ITEM)
      {
        if (control == _facadeLayout)
        {
          int selectedListItemIndex = _facadeLayout.SelectedListItemIndex;
          if (selectedListItemIndex != -1)
          {
            _selectedCamera = ((Content.Camera) _facadeLayout[selectedListItemIndex].TVTag);
            PlayLiveViewFromSelectedCamera();
          }
        }
      }

      if (_selectedCamera != null)
      {
        if (control == _govideo)
        {
          Cleanup();
          GUIPropertyManager.SetProperty("#ispy_oid", _selectedCamera.Id.ToString());
          var cam = new ispy_content();
          var windowcam = (GUIWindow) cam;
          GUIWindowManager.Add(ref windowcam);
          cam.Init();
          GUIWindowManager.ActivateWindow(cam.GetID, false);
        }
        string cmd = "";
        if (control == _cmdrecord)
        {
          cmd = "record";
        }
        if (control == _cmdstoprecord)
        {
          cmd = "recordoff";
        }
        if (control == _cmdenable)
        {
          cmd = "bringonline";
        }
        if (control == _cmddisable)
        {
          cmd = "takeoffline";
        }
        if (cmd != "")
        {
          string url = "http://" + Server.ipaddress + ":" + Server.port + "/" + cmd + "?ot=2&oid=" +
                       _selectedCamera.Id;
          Log.Debug(url);
          string r = Utility.Tools.GetFullUrlContent(url, 3);

          using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(MessageBoxId))
          {
            messageBox.SetHeading("Command Executed");
            messageBox.SetLines("Response: " + r);
            messageBox.DoModal(GUIWindowManager.ActiveWindow);
          }
        }
      }
    }

    /// <summary>
    /// Plays rolling live view from specified list item index representing a camera.
    /// </summary>
    private void PlayLiveViewFromSelectedCamera()
    {
      try
      {
        if (_streamer != null)
        {
          _streamer.SignalToStop();
          _streamer.WaitForStop();
          _streamer = null;
        }

        if (_selectedCamera != null)
        {
          string url = string.Format("http://{0}:{1}/mjpegfeed?oid={2}&size=1280x720", Server.ipaddress, Server.port,
            _selectedCamera.Id);

          _streamer = new MJPEGStream(url) {RequestTimeout = 4000};
          _streamer.NewFrame += StreamerNewFrame;
          _streamer.VideoSourceError += StreamerVideoSourceError;
          _streamer.PlayingFinished += StreamerPlayingFinished;
          _streamer.Start();
        }
      }
      catch (Exception e)
      {
        Log.Error("Error during PlayLiveViewFromSelectedCamera");
        Log.Error(e.Message);
      }
    }

    void StreamerPlayingFinished(object sender, ReasonToFinishPlaying reason)
    {
      lock (this)
      {
        GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 1280, 720);
        _videoPlayer.SetFileName("[ispy:offline]");
        _videoPlayer.UpdateLayout();
      }
    }

    void StreamerVideoSourceError(object sender, VideoSourceErrorEventArgs eventArgs)
    {
      lock (this)
      {
        GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 1280, 720);
        _videoPlayer.SetFileName("[ispy:offline]");
        _videoPlayer.UpdateLayout();
      }
    }

    void StreamerNewFrame(object sender, NewFrameEventArgs eventArgs)
    {

      lock (this)
      {
        if (_framecount > 0)
        {
          _videoPlayer.RemoveMemoryImageTexture();
          GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
        }
        _framecount++;
        try
        {
          GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
            eventArgs.Frame.Height);
        }
        catch
        {

        }
        _videoPlayer.SetFileName("[ispy:video" + _framecount + "]");
        _videoPlayer.UpdateLayout();
      }

    }

    public override int GetID
    {
      get { return WindowId; }
      set { }
    }
  }
}