﻿#region Copyright (C) 2005-2012 Team MediaPortal

// Copyright (C) 2005-2011 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Timers;
using iSpyPlugin.Configuration;
using MediaPortal.Dialogs;
using MediaPortal.GUI.Library;

namespace iSpyPlugin
{
  public class ispy_cameras_grid : GUIWindow
  {
    private const int WindowId = 157810;
    private configIspy _server = null;

    public configIspy Server
    {
      get
      {
        return _server ?? (_server = ServerList.Servers.ispy.SingleOrDefault(
          p => p.ipaddress == GUIPropertyManager.GetProperty("#ispyserver")));
      }
      set { _server = value; }
    }
    private System.Timers.Timer _tmrRoll;
    private int _cameraindex;

    const int MessageBoxId = (int) GUIWindow.Window.WINDOW_DIALOG_OK;

    private List<Content.Camera> _liveCameras;

    [SkinControl(500)]
    private readonly GUIImage _videoPlayer0 = null;
    [SkinControl(501)]
    private readonly GUIImage _videoPlayer1 = null;
    [SkinControl(502)]
    private readonly GUIImage _videoPlayer2 = null;
    [SkinControl(503)]
    private readonly GUIImage _videoPlayer3 = null;


    [SkinControl(2)] private readonly GUIButtonControl _govideo = null;

    [SkinControl(3)] private readonly GUIButtonControl _cmdrecord = null;

    [SkinControl(4)] private readonly GUIButtonControl _cmdstoprecord = null;

    [SkinControl(5)] private readonly GUIButtonControl _cmdenable = null;

    [SkinControl(6)] private readonly GUIButtonControl _cmddisable = null;

    public override bool Init()
    {

      return Load(GUIGraphicsContext.Skin + @"\ispy_cameras_grid2x2.xml");
    }

    protected override void OnPreviousWindow()
    {
      Cleanup();
      base.OnPreviousWindow();
    }

    private void Cleanup()
    {
      try
      {
        if (_tmrRoll != null)
        {
          _tmrRoll.Stop();
          _tmrRoll.Dispose();
          _tmrRoll = null;
        }
      }
      catch (Exception e)
      {
        Log.Error("Error during cleanup");
        Log.Error(e.Message);
      }
    }

    protected override void OnPageLoad()
    {
      _server = null;
      if (ServerList.Servers.ispy.Length > 0)
      {
        _server = ServerList.Servers.ispy[0];
      }

      bool passed = false;
      var cams = new List<Content.Camera>();
      Cleanup();
      try
      {
        cams = Content.Cameras(Server);
        if (cams.Count == 0)
        {
          using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(MessageBoxId))
          {
            messageBox.SetHeading("No cameras found at " + Server.ipaddress);
            messageBox.SetLines("To add cameras, add them to iSpy and then try again.");
            messageBox.DoModal(GUIWindowManager.ActiveWindow);
          }
        }
        else
        {
          passed = true;
        }
      }
      catch (Exception ex)
      {
        using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(MessageBoxId))
        {
          messageBox.SetHeading("Error accessing iSpy at " + Server.ipaddress);
          messageBox.SetLines("Check your IP address and access permissions and try again.\n" +
                              ex.Message);
          messageBox.DoModal(GUIWindowManager.ActiveWindow);
        }
      }

      if (passed)
      {
        GUIWaitCursor.Init();
        GUIWaitCursor.Show();

        _liveCameras = Content.Cameras(_server).Where(p => p.Online).ToList();
        CreateGridImages();

        _tmrRoll = new System.Timers.Timer(5000);
        _tmrRoll.Elapsed += TmrRollElapsed;
        _tmrRoll.Start();

        GUIWaitCursor.Hide();
      }

      base.OnPageLoad();
    }

    protected override void OnPageDestroy(int newWindowId)
    {
      base.OnPageDestroy(newWindowId);
    }

    public override void Process()
    {
      lock (this)
      {
        base.Process();
      }
    }

    public override void DeInit()
    {
      Cleanup();
      base.DeInit();
    }

    private void CreateGridImages()
    {
      int count = 0;
      List<Image> images = new List<Image>();

      // Retrieve images
      foreach (var camera in _liveCameras)
      {
        string url = string.Format("http://{0}:{1}/livefeed?oid={2}&size=1280x720", Server.ipaddress, Server.port,
          camera.Id);
        var img =
          Utility.Tools.GetImageFromUrl(url);

        images.Add(img);
      }

      // Set images - makes it faster to update after retrieval (less delay)
      foreach (var camera in _liveCameras)
      {
        if (count == 0)
        {
          if (!string.IsNullOrEmpty(_videoPlayer0.FileName))
          {
            _videoPlayer0.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[" + camera.Ident + "]");
          }

          try
          {
            GUITextureManager.LoadFromMemory(images[count], "[" + camera.Ident + "]", 0, 1280, 720);
          }
          catch
          {

          }
          _videoPlayer0.SetFileName("[" + camera.Ident + "]");
          _videoPlayer0.UpdateLayout();
        }
        if (count == 1)
        {
          if (!string.IsNullOrEmpty(_videoPlayer1.FileName))
          {
            _videoPlayer1.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[" + camera.Ident + "]");
          }

          try
          {
            GUITextureManager.LoadFromMemory(images[count], "[" + camera.Ident + "]", 0, 1280, 720);
          }
          catch
          {

          }
          _videoPlayer1.SetFileName("[" + camera.Ident + "]");
          _videoPlayer1.UpdateLayout();
        }
        if (count == 2)
        {
          if (!string.IsNullOrEmpty(_videoPlayer2.FileName))
          {
            _videoPlayer2.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[" + camera.Ident + "]");
          }

          try
          {
            GUITextureManager.LoadFromMemory(images[count], "[" + camera.Ident + "]", 0, 1280, 720);
          }
          catch
          {

          }
          _videoPlayer2.SetFileName("[" + camera.Ident + "]");
          _videoPlayer2.UpdateLayout();
        }
        if (count == 3)
        {
          if (!string.IsNullOrEmpty(_videoPlayer3.FileName))
          {
            _videoPlayer3.RemoveMemoryImageTexture();
            GUITextureManager.ReleaseTexture("[" + camera.Ident + "]");
          }

          try
          {
            GUITextureManager.LoadFromMemory(images[count], "[" + camera.Ident + "]", 0, 1280, 720);
          }
          catch
          {

          }
          _videoPlayer3.SetFileName("[" + camera.Ident + "]");
          _videoPlayer3.UpdateLayout();
        }
        count++;
      }
    }

    void TmrRollElapsed(object sender, ElapsedEventArgs e)
    {
      CreateGridImages();
    }

    public override
      int GetID
    {
      get { return WindowId; }
      set { }
    }
  }
}