﻿#region Copyright (C) 2005-2012 Team MediaPortal

// Copyright (C) 2005-2011 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using iSpyPlugin.Configuration;
using MediaPortal.GUI.Library;

using Action = MediaPortal.GUI.Library.Action;

namespace iSpyPlugin
{
    public class ispy_content : GUIWindow
    {
        private const int WindowId = 157802;
        private const int MaxFiles = 30;

        private configIspy _server;
        private IMoviePlayer _player;

        private List<Preview> _previews;
        public configIspy Server
        {
            get
            {
                return _server ?? (_server = ServerList.Servers.ispy.SingleOrDefault(
                    p => p.ipaddress == GUIPropertyManager.GetProperty("#ispyserver")));
            }
            set { _server = value; }
        }

        /// <summary>
        /// The facade layout containing cameras.
        /// </summary>
        [SkinControl(50)]
        private readonly GUICoverFlow _coverLayout = null;

        private int CameraID
        {
            get { return Convert.ToInt32(GUIPropertyManager.GetProperty("#ispy_oid")); }
        }

        public override bool Init()
        {
            _player = new MoviePlayer(new MediaPortalPlayer());
            return Load(GUIGraphicsContext.Skin + @"\ispy_content.xml");
        }

        protected override void  OnPreviousWindow()
        {
            Cleanup();
 	        base.OnPreviousWindow();
        }

        private void Cleanup()
        {
        }

        protected override void  OnPageLoad()
        {
            _server = null;
            //load filmstrip
            _previews = new List<Preview>();
            string url = "http://" + Server.ipaddress + ":" + Server.port + "/getcontentlist?ot=2&oid=" + CameraID + "&page=0&ordermode=0&pagesize="+MaxFiles;
            string pl = Utility.Tools.GetFullUrlContent(url, 5);

            var pll = pl.Split(',');
            foreach (string s2 in pll)
            {
                if (!String.IsNullOrEmpty(s2))
                {
                    var p = new Preview();
                    var cfg = s2.Split('|');
                    p.VideoFileName = cfg[0];
                    double d;
                    if (double.TryParse(cfg[2], out d))
                        p.DurationSeconds = Convert.ToInt32(d);
                    else
                    {
                        p.DurationSeconds = 0;
                    }

                    string dt = cfg[0].Substring(cfg[0].IndexOf("_") + 1);
                    dt = dt.Substring(0, dt.IndexOf("."));
                    var dt2 = dt.Split('_');
                    var d1 = dt2[0].Split('-');
                    int year = Convert.ToInt32(d1[0]);
                    int month = Convert.ToInt32(d1[1]);
                    int day = Convert.ToInt32(d1[2]);
                    var d2 = dt2[1].Split('-');
                    int hour = Convert.ToInt32(d2[0]);
                    int minute = Convert.ToInt32(d2[1]);
                    int second = Convert.ToInt32(d2[2]);
                    p.Created = new DateTime(year, month, day, hour, minute, second);
                    p.ThumbFileName = p.VideoFileName.Replace(".mp4", "_large.jpg");
                    //p.PreviewFileName = p.VideoFileName.Replace(".mp4", "_large.jpg");
                    _previews.Add(p);
                }
            }
            
            foreach (var pre in _previews)
            {
                _coverLayout.Add(CreateListItemFrom(pre));
            }
            string si = GUIPropertyManager.GetProperty("#ispycard");
            if (!String.IsNullOrEmpty(si))
            {
                int i = Convert.ToInt32(si);
                if (i<_previews.Count)
                    _coverLayout.SelectCard(i);
            }
            base.OnPageLoad();
        }


        ///<summary>
        /// Creates a list item representing a video file.
        /// </summary>
        /// <param name="preview">The preview.</param>
        /// <returns>A list item representing a preview.</returns>
        private GUIListItem CreateListItemFrom(Preview preview)
        {
            var listItem = new GUIListItem
            {
                Label = preview.Created.ToString("{0:G}"),
                //IconImage = ioService.Value.CameraIconFileName,
                MusicTag = preview
            };
            var img = Utility.Tools.GetImageFromUrl("http://"+Server.ipaddress+":"+Server.port + "/loadimage?oid=" + CameraID + "&fn=" + preview.ThumbFileName);

            

            if (img != null)
            {
                Image.GetThumbnailImageAbort myCallback = ThumbnailCallback;
                Image myThumbnail = img.GetThumbnailImage(320, 240, myCallback, IntPtr.Zero);

                GUITextureManager.LoadFromMemory(myThumbnail, "[" + preview.ThumbFileName + "]", 0x00000000, 320, 240);
                listItem.ThumbnailImage = "[" + preview.ThumbFileName + "]";
                myThumbnail.Dispose();
                img.Dispose();    
            }

            return listItem;
        }

        private bool ThumbnailCallback()
        {
            return false;
        }

        #region FileBrowser

        internal class Preview
        {
            public DateTime Created;
            public string VideoFileName;
            public string ThumbFileName;
            public int DurationSeconds;
        }
        #endregion

        public override void Process()
        {
            lock (this)
            {
                base.Process();
            }
        }

        public override void DeInit()
        {
            Cleanup();
            base.DeInit();
        }



        /// <summary>
        /// Called when control was clicked.
        /// </summary>
        /// <param name="controlId">The control id.</param>
        /// <param name="control">The control.</param>
        /// <param name="actionType">Type of the action.</param>
        protected override void OnClicked(int controlId, GUIControl control, Action.ActionType actionType)
        {
            base.OnClicked(controlId, control, actionType);

            
            if (control == _coverLayout)
            {
                int selectedListItemIndex = _coverLayout.SelectedListItemIndex;
                if (selectedListItemIndex != -1)
                {
                    var c = ((Preview)_coverLayout[selectedListItemIndex].MusicTag);
                    GUIPropertyManager.SetProperty("#ispycard", selectedListItemIndex.ToString());
                    _player.PlayMovie("http://"+Server.ipaddress+":"+Server.port+"/loadclip.mp4?oid=" + CameraID + "&ot=2&fn=" + c.VideoFileName, String.Format("{0:G}",c.Created));

                }
            }
         
        }
       

        public override int GetID
        {
            get { return WindowId; }
            set { }
        }
    }
}