﻿#region Copyright (C) 2005-2012 Team MediaPortal

// Copyright (C) 2005-2011 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Timers;
using iSpyPlugin.Configuration;
using MediaPortal.Configuration;
using MediaPortal.Dialogs;
using MediaPortal.GUI.Library;

using Action = MediaPortal.GUI.Library.Action;

namespace iSpyPlugin
{
  [PluginIcons(
    "iSpyPlugin.icon.png",
    "iSpyPlugin.icon_disabled.png")]
  public class ispy_servers : GUIWindow, ISetupForm
  {
    private const int WindowId = 157800;
    //private Thread ThumbUpdater;
    private MJPEGStream _streamer;
    private configIspy _selectedServer;
    private Timer _tmrRoll;
    private int _cameraindex;
    private Int64 _framecount;
    private List<Content.Camera> _liveCameras;

    /// <summary>
    /// The facade layout containing cameras.
    /// </summary>
    [SkinControl(50)] private readonly GUIFacadeControl _facadeLayout = null;

    [SkinControl(2)] private readonly GUIButtonControl _goCameras = null;
    [SkinControl(3)] private readonly GUIButtonControl _goGrid = null;

    [SkinControl(500)] private readonly GUIImage _videoPlayer = null;

    public override bool Init()
    {
      return Load(GUIGraphicsContext.Skin + @"\ispy_servers.xml");
    }

    protected override void OnPreviousWindow()
    {

      Cleanup();
      base.OnPreviousWindow();
    }

    private void Cleanup()
    {

      if (_tmrRoll != null)
      {
        _tmrRoll.Stop();
        _tmrRoll.Dispose();
        _tmrRoll = null;
      }

      if (_streamer != null)
      {
        _streamer.SignalToStop();
        _streamer.WaitForStop();
        _streamer = null;
      }

    }

    protected override void OnPageLoad()
    {
      const int messageBoxId = (int) Window.WINDOW_DIALOG_OK;

      bool first = true;
      if (ServerList.Servers.ispy.Length > 0)
      {
        bool passed = false;
        foreach (var server in ServerList.Servers.ispy)
        {
          try
          {
            int cams = Content.Cameras(server, true).Count;

            if (cams == 0)
            {
              using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(messageBoxId))
              {
                messageBox.SetHeading("No cameras found at " + server.ipaddress);
                messageBox.SetLines("To add cameras, add them to iSpy and then try again.");
                messageBox.DoModal(GUIWindowManager.ActiveWindow);
              }
            }
            else
            {
              passed = true;
              _facadeLayout.Add(CreateListItemFrom(server));
              if (first)
                _selectedServer = server;
              first = false;
            }
          }
          catch (Exception ex)
          {
            using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(messageBoxId))
            {
              messageBox.SetHeading("Error accessing iSpy at " + server.ipaddress);
              messageBox.SetLines("Check your IP address and access permissions and try again.\n" +
                                  ex.Message);
              messageBox.DoModal(GUIWindowManager.ActiveWindow);
            }
          }
        }

        if (passed)
        {
          GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
          SetSelectedServer();
          _liveCameras = Content.Cameras(_selectedServer).Where(p => p.Online).ToList();
          PlayLiveViewFromSelectedServer();

          _tmrRoll = new Timer(5000);
          _tmrRoll.Elapsed += TmrRollElapsed;
          _tmrRoll.Start();
        }
      }
      else
      {
        // Display message to user about how to add cameras to the plugin
        using (var messageBox = (GUIDialogOK) GUIWindowManager.GetWindow(messageBoxId))
        {
          messageBox.SetHeading("No iSpy Servers Listed");
          messageBox.SetLines(
            "To add servers, exit MediaPortal and\nconfigure the plugin in the MediaPortal\nConfiguration application.");
          messageBox.DoModal(GUIWindowManager.ActiveWindow);
        }
      }

      base.OnPageLoad();
    }

    void TmrRollElapsed(object sender, ElapsedEventArgs e)
    {
      _cameraindex++;
      PlayLiveViewFromSelectedServer();
    }

    public override void Process()
    {
      lock (this)
      {
        base.Process();
      }
    }

    public override void DeInit()
    {
      Cleanup();
      base.DeInit();
    }

    /// <summary>
    /// Sets the selected server.
    /// </summary>
    private void SetSelectedServer()
    {
      // Rather ugly way of setting the selected camera

      for (int index = 0; index < _facadeLayout.Count; index++)
      {
        var server = (configIspy) _facadeLayout[index].MusicTag;

        if (server.ipaddress == _selectedServer.ipaddress)
        {
          // Set the last selected camera
          GUIControl.SelectItemControl(GetID, _facadeLayout.GetID, index);
          break;
        }
      }

    }

    /// <summary>
    /// Creates a list item representing a server.
    /// </summary>
    /// <param name="server">The server.</param>
    /// <returns>A list item representing a server.</returns>
    private GUIListItem CreateListItemFrom(configIspy server)
    {
      var listItem = new GUIListItem
      {
        Label = server.ipaddress,
        //IconImage = ioService.Value.CameraIconFileName,
        MusicTag = server
      };

      // Register for selected item changes

      return listItem;
    }


    /// <summary>
    /// Called when control was clicked.
    /// </summary>
    /// <param name="controlId">The control id.</param>
    /// <param name="control">The control.</param>
    /// <param name="actionType">Type of the action.</param>
    protected override void OnClicked(int controlId, GUIControl control, Action.ActionType actionType)
    {
      base.OnClicked(controlId, control, actionType);

      if (actionType == Action.ActionType.ACTION_SELECT_ITEM && control != _goCameras)
      {
        Cleanup();
        GUIPropertyManager.SetProperty("#ispyserver", _selectedServer.ipaddress);
        var cam = new ispy_cameras();
        var windowcam = (GUIWindow) cam;
        GUIWindowManager.Add(ref windowcam);
        cam.Init();
        GUIWindowManager.ActivateWindow(cam.GetID, false);
        if (control == _facadeLayout)
        {
          int selectedListItemIndex = _facadeLayout.SelectedListItemIndex;
          if (selectedListItemIndex != -1)
          {
            _tmrRoll.Stop();
            _selectedServer = ((configIspy) _facadeLayout[selectedListItemIndex].MusicTag);

            _liveCameras = Content.Cameras(_selectedServer).Where(p => p.Online).ToList();
            _cameraindex = 0;
            PlayLiveViewFromSelectedServer();
            _tmrRoll.Start();
          }
        }
      }
      else if (control == _goCameras)
      {
        try
        {
          Cleanup();
          _tmrRoll.Stop();
        }
        catch (Exception)
        {
        }

        GUIPropertyManager.SetProperty("#ispyserver", _selectedServer.ipaddress);
        var cam = new ispy_cameras();
        var windowcam = (GUIWindow) cam;
        GUIWindowManager.Add(ref windowcam);
        cam.Init();
        GUIWindowManager.ActivateWindow(cam.GetID, false);
      }
      else if (control == _goGrid)
      {
        try
        {
          Cleanup();
          _tmrRoll.Stop();
        }
        catch (Exception)
        {
        }

        GUIPropertyManager.SetProperty("#ispyserver", _selectedServer.ipaddress);
        var grid = new ispy_cameras_grid();
        var windowgrid = (GUIWindow)grid;
        GUIWindowManager.Add(ref windowgrid);
        grid.Init();
        GUIWindowManager.ActivateWindow(grid.GetID, false);
      }
    }



    /// <summary>
    /// Plays rolling live view from specified list item index representing a camera.
    /// </summary>
    private void PlayLiveViewFromSelectedServer()
    {
      if (_streamer != null)
      {
        _streamer.SignalToStop();
        _streamer.WaitForStop();
        _streamer = null;
      }

      int c = _liveCameras.Count;
      if (_cameraindex >= c)
        _cameraindex = 0;

      if (c > _cameraindex)
      {
        string url = "http://" + _selectedServer.ipaddress + ":" + _selectedServer.port + "/mjpegfeed?oid=" +
                     _liveCameras[_cameraindex].Id + "&size=640x480";

        _streamer = new MJPEGStream(url) {RequestTimeout = 4000};
        _streamer.NewFrame += StreamerNewFrame;
        _streamer.VideoSourceError += StreamerVideoSourceError;
        _streamer.PlayingFinished += StreamerPlayingFinished;
        _streamer.Start();
      }
    }

    void StreamerPlayingFinished(object sender, ReasonToFinishPlaying reason)
    {
      //_videoPlayer.SetFileName("[ispy:offline]");
      //_videoPlayer.UpdateLayout();
    }

    void StreamerVideoSourceError(object sender, VideoSourceErrorEventArgs eventArgs)
    {
      lock (this)
      {
        GUITextureManager.LoadFromMemory(Properties.Resources.offline, "[ispy:offline]", 0, 640, 480);
        _videoPlayer.SetFileName("[ispy:offline]");
        _videoPlayer.UpdateLayout();
      }
    }

    void StreamerNewFrame(object sender, NewFrameEventArgs eventArgs)
    {

      lock (this)
      {
        if (_framecount > 0)
        {
          _videoPlayer.RemoveMemoryImageTexture();
          GUITextureManager.ReleaseTexture("[ispy:video" + _framecount + "]");
        }
        _framecount++;
        try
        {
          GUITextureManager.LoadFromMemory(eventArgs.Frame, "[ispy:video" + _framecount + "]", 0, eventArgs.Frame.Width,
            eventArgs.Frame.Height);
        }
        catch
        {

        }
        _videoPlayer.SetFileName("[ispy:video" + _framecount + "]");
        _videoPlayer.UpdateLayout();
      }
    }

    public override int GetID
    {
      get { return WindowId; }
      set { }
    }

    #region ISetupForm Members

    public string PluginName()
    {
      return "iSpy";
    }


    // Returns the description of the plugin is shown in the plugin menu

    public string Description()
    {
      return "Content Viewer for iSpyConnect (http://www.ispyconnect.com)";
    }


    // Returns the author of the plugin which is shown in the plugin menu

    public string Author()
    {
      return "Sean Tearney";
    }


    // show the setup dialog

    public void ShowPlugin()
    {
      NetworkConfiguration.SetAllowUnsafeHeaderParsing20();
      var c = new Configure();
      c.ShowDialog();
      c.Dispose();
    }


    // Indicates whether plugin can be enabled/disabled

    public bool CanEnable()
    {
      return true;
    }


    // Get Windows-ID

    public int GetWindowId()
    {
      // WindowID of windowplugin belonging to this setup

      // enter your own unique code

      return WindowId;
    }


    // Indicates if plugin is enabled by default;

    public bool DefaultEnabled()
    {
      return true;
    }


    // indicates if a plugin has it's own setup screen

    public bool HasSetup()
    {
      return true;
    }


    /// <summary>
    /// If the plugin should have it's own button on the main menu of MediaPortal then it
    /// should return true to this method, otherwise if it should not be on home
    /// it should return false
    /// </summary>
    /// <param name="strButtonText">text the button should have</param>
    /// <param name="strButtonImage">image for the button, or empty for default</param>
    /// <param name="strButtonImageFocus">image for the button, or empty for default</param>
    /// <param name="strPictureImage">subpicture for the button or empty for none</param>
    /// <returns>true : plugin needs it's own button on home
    /// false : plugin does not need it's own button on home</returns>
    public bool GetHome(out string strButtonText, out string strButtonImage,
      out string strButtonImageFocus, out string strPictureImage)
    {
      strButtonText = PluginName();

      strButtonImage = String.Empty;

      strButtonImageFocus = String.Empty;

      strPictureImage = "hover_iSpy.png";

      return true;
    }

    #endregion
  }
}